==> odoo_18.0.20251031.zip: odoo-18.0.post20251031/odoo/addons/html_editor/static/src/core/overlay_plugin.js <==
import { markRaw, EventBus } from "@odoo/owl";
import { Plugin } from "../plugin";
import { EditorOverlay } from "./overlay";
import { throttleForAnimation } from "@web/core/utils/timing";
import { findUpTo } from "@html_editor/utils/dom_traversal";

/**
 * @typedef { Object } OverlayShared
 * @property { OverlayPlugin['createOverlay'] } createOverlay
 */

/**
 * Provides the following feature:
 * - adding a component in overlay above the editor, with proper positioning
 */
export class OverlayPlugin extends Plugin {
    static id = "overlay";
    static dependencies = ["history"];
    static shared = ["createOverlay"];
    resources = {
        step_added_handlers: this.getScrollContainer.bind(this),
    };

    overlays = [];

    setup() {
        this.iframe = this.document.defaultView.frameElement;
        this.topDocument = this.iframe?.ownerDocument || this.document;
        this.container = this.getScrollContainer();
        this.throttledUpdateContainer = throttleForAnimation(() => {
            this.container = this.getScrollContainer();
        });
        this.addDomListener(this.topDocument.defaultView, "resize", this.throttledUpdateContainer);
    }

    destroy() {
        this.throttledUpdateContainer.cancel();
        super.destroy();
        for (const overlay of this.overlays) {
            overlay.close();
        }
    }

    /**
     * Creates an overlay component and adds it to the list of overlays.
     *
     * @param {Function} Component
     * @param {Object} [props={}]
     * @param {Object} [options]
     * @returns {Overlay}
     */
    createOverlay(Component, props = {}, options) {
        const overlay = new Overlay(this, Component, () => this.container, props, options);
        this.overlays.push(overlay);
        return overlay;
    }

    getScrollContainer() {
        const isScrollable = (element) =>
            element.scrollHeight > element.clientHeight &&
            ["auto", "scroll"].includes(getComputedStyle(element).overflowY);

        return (
            findUpTo(this.iframe || this.editable, null, isScrollable) ||
            this.topDocument.documentElement
        );
    }
}

export class Overlay {
    constructor(plugin, C, getContainer, props, options) {
        this.plugin = plugin;
        this.C = C;
        this.editorOverlayProps = props;
        this.options = options;
        this.isOpen = false;
        this._remove = null;
        this.component = null;
        this.bus = new EventBus();
        this.getContainer = getContainer;
    }

    /**
     * @param {Object} options
     * @param {HTMLElement | null} [options.target] for the overlay.
     *  If null or undefined, the current selection will be used instead
     * @param {any} [options.props] overlay component props
     */
    open({ target, props }) {
        if (this.isOpen) {
            this.updatePosition();
        } else {
            this.isOpen = true;
            const selection = this.plugin.editable.ownerDocument.getSelection();
            let initialSelection;
            if (selection && selection.type !== "None") {
                initialSelection = {
                    range: selection.getRangeAt(0),
                };
            }
            this._remove = this.plugin.services.overlay.add(
                EditorOverlay,
                markRaw({
                    ...this.editorOverlayProps,
                    Component: this.C,
                    editable: this.plugin.editable,
                    props,
                    target,
                    initialSelection,
                    bus: this.bus,
                    getContainer: this.getContainer,
                    close: this.close.bind(this),
                    isOverlayOpen: this.isOverlayOpen.bind(this),
                    history: {
                        enableObserver: this.plugin.dependencies.history.enableObserver,
                        disableObserver: this.plugin.dependencies.history.disableObserver,
                    },
                }),
                {
                    ...this.options,
                }
            );
        }
    }

    close() {
        this.isOpen = false;
        if (this._remove) {
            this._remove();
        }
    }

    isOverlayOpen() {
        return this.isOpen;
    }

    updatePosition() {
        this.bus.trigger("updatePosition");
    }
}

==> odoo_19.0.20260130.zip: odoo-19.0.post20260130/odoo/addons/html_editor/static/src/core/overlay_plugin.js <==
import { markRaw, EventBus } from "@odoo/owl";
import { Plugin } from "../plugin";
import { EditorOverlay } from "./overlay";

/**
 * @typedef { Object } OverlayShared
 * @property { OverlayPlugin['createOverlay'] } createOverlay
 */

/**
 * Provides the following feature:
 * - adding a component in overlay above the editor, with proper positioning
 */
export class OverlayPlugin extends Plugin {
    static id = "overlay";
    static dependencies = ["history", "selection"];
    static shared = ["createOverlay"];

    overlays = [];

    destroy() {
        super.destroy();
        for (const overlay of this.overlays) {
            overlay.close();
        }
    }

    /**
     * Creates an overlay component and adds it to the list of overlays.
     *
     * @param {Function} Component
     * @param {Object} [props={}]
     * @param {Object} [options]
     * @returns {Overlay}
     */
    createOverlay(Component, props = {}, options) {
        const overlay = new Overlay(this, Component, props, options);
        this.overlays.push(overlay);
        return overlay;
    }
}

export class Overlay {
    constructor(plugin, C, props, options) {
        this.plugin = plugin;
        this.C = C;
        this.editorOverlayProps = props;
        this.options = options;
        this.isOpen = false;
        this._remove = null;
        this.component = null;
        this.bus = new EventBus();
    }

    /**
     * @param {Object} options
     * @param {HTMLElement | null} [options.target] for the overlay.
     *  If null or undefined, the current selection will be used instead
     * @param {any} [options.props] overlay component props
     */
    open({ target, props }) {
        if (this.isOpen) {
            this.updatePosition();
        } else {
            this.isOpen = true;
            const selection = this.plugin.editable.ownerDocument.getSelection();
            let initialSelection;
            if (selection && selection.type !== "None") {
                initialSelection = {
                    range: selection.getRangeAt(0),
                };
            }
            this._remove = this.plugin.services.overlay.add(
                EditorOverlay,
                markRaw({
                    ...this.editorOverlayProps,
                    Component: this.C,
                    editable: this.plugin.editable,
                    props,
                    target,
                    initialSelection,
                    bus: this.bus,
                    close: this.close.bind(this),
                    isOverlayOpen: this.isOverlayOpen.bind(this),
                    shared: {
                        ignoreDOMMutations: this.plugin.dependencies.history.ignoreDOMMutations,
                        getSelectionData: this.plugin.dependencies.selection.getSelectionData,
                    },
                }),
                {
                    ...this.options,
                }
            );
        }
    }

    close() {
        this.isOpen = false;
        if (this._remove) {
            this._remove();
        }
    }

    isOverlayOpen() {
        return this.isOpen;
    }

    updatePosition() {
        this.bus.trigger("updatePosition");
    }
}
